SystemOrganization addCategory: #'Magma server requests'!

Object subclass: #MagmaRequestResponder
	instanceVariableNames: 'repositoryController'
	classVariableNames: 'ValidSelectors'
	poolDictionaries: ''
	category: 'Magma server requests'!

MagmaRequestResponder class
	instanceVariableNames: 'responders'!

!MagmaRequestResponder commentStamp: 'kph 6/15/2007 04:00' prior: 0!
This class implements the server side responder.

To test it from the client.

Q2ServerRequest counterIncrementAt: #myKey.   
Q2ServerRequest counterValueAt: #myKey.

To test an error raised on the server:

Q2ServerRequest doesNotUnderstand: (Message selector: #error: argument: 'my error').

To test a DNU raised on the server:

Q2ServerRequest willNotUnderstand.

TODO: the counter is not yet persisted!

MagmaRequestResponder class
	instanceVariableNames: 'responders'!

MagmaRequestResponder subclass: #MagmaCounterResponder
	instanceVariableNames: 'counters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma server requests'!

!MagmaCounterResponder class methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 05:00'!
requestsSupported

	^ #( counterIncrementAt: counterValueAt: #counters)! !

!MagmaCounterResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 03:42'!
counterIncrementAt: key

	^ self counters at: key put: (self counterValueAt: key) + 1! !

!MagmaCounterResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 03:42'!
counterResetAt: key

	^ self counters removeKey: key  ! !

!MagmaCounterResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 03:41'!
counters

	^ counters ifNil:[ counters := IdentityDictionary new ]! !

!MagmaCounterResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 03:41'!
counterValueAt: key

	^ self counters at: key ifAbsentPut:[ 1 ]! !

MagmaRequestResponder subclass: #MagmaDictionaryResponder
	instanceVariableNames: 'dict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma server requests'!

!MagmaDictionaryResponder class methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 05:28'!
requestsSupported

	^ #( dataAt: dataAt:put: keys)! !

!MagmaDictionaryResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 05:27'!
dataAt: key put: v

	^ self data at: key put: v! !

!MagmaDictionaryResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 05:27'!
dataAt: key

	^ self data at: key ifAbsent: nil! !

!MagmaDictionaryResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 05:26'!
data

	^ dict! !

!MagmaDictionaryResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 05:28'!
keys

	^ dict keys! !

!MagmaRequestResponder class methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:32'!
initialize

	ValidSelectors := IdentityDictionary new.

	self allSubclassesDo: [ :sc |
		sc requestsSupported do: [ :selector |
				ValidSelectors at: selector put: sc
		]	
	]
	! !

!MagmaRequestResponder class methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:49'!
newFor: rc

	^ self basicNew setRepositoryController: rc; initialize; yourself! !

!MagmaRequestResponder class methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:30'!
responderFor: aRepositoryController playRequests: theRequests

	| theRespondingClass theResponder results  |

	results := OrderedCollection new.

	theRequests do: [ :req |
	
			theRespondingClass := ValidSelectors at: req selector
				ifAbsent: [ ^ self  error: 'request not supported' ].

			theResponder := theRespondingClass responders 
				at:  aRepositoryController localLocation pathName 
				ifAbsentPut: [ theRespondingClass newFor: aRepositoryController ].
	 

		theResponder wantsControl ifTrue: [ ^ theResponder playRequests: theRequests ].
	
		results add: 
			([req  sendTo: theResponder] 
				on: Error 
				do: [ :ex | theResponder respondWithException: ex ])	
	].

	^ results size = 1 ifTrue: [ results first ] ifFalse: [ results ] ! !

!MagmaRequestResponder class methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 03:33'!
responders
	^ responders ifNil: [ responders := Dictionary new ]! !

!MagmaRequestResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 02:37'!
repositoryController

	^ repositoryController  ! !

!MagmaRequestResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 03:01'!
respondWithException: anException

	| signalableErrorForTheClient |
	signalableErrorForTheClient :=  anException maOriginalClass new 
		copyAttributes:
			(anException maOriginalClass allInstVarNames copyWithoutAll: #('signalContext' 'handlerContext' 'outerContext'))
		from: anException.
 
	^ signalableErrorForTheClient! !

!MagmaRequestResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:34'!
setRepositoryController: rp  

	repositoryController := rp.
	 ! !

MagmaRequestResponder subclass: #MagmaServerBackdoorResponder
	instanceVariableNames: 'counters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma server requests'!

!MagmaServerBackdoorResponder class methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:33'!
requestsSupported

	^ #( #login:password: )! !

!MagmaServerBackdoorResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:28'!
login: user password: pass

	^ (user = 'admin') & (pass = 'magma rocks')! !

!MagmaServerBackdoorResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:24'!
playRequests: requests

	"highly dangerous and hence useful"
	
	| remainingRequests authenticated |

	"first request must be #login:password:"

	authenticated := requests first sendTo: self.

	authenticated ifFalse: [ ^ 'Invalid user/password' ].

	remainingRequests := requests copyWithoutFirst.
 
	^ [ remainingRequests inject: self into: [  :receiver  :request | request sendTo: receiver ]   ] 
			on: Error do: [ :ex |  self respondWithException: ex ] ! !

!MagmaServerBackdoorResponder methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 03:17'!
wantsControl

	^ true

	! !

MagmaRepositoryRequest subclass: #MagmaServerRequest
	instanceVariableNames: 'requests submitReceiver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma server requests'!

!MagmaServerRequest commentStamp: 'kph 6/16/2007 03:54' prior: 0!
usage:
 
1) To make a simple request

(aMagmaSession serverRequest counterValueAt: #myKey) submit.

2) Or for delayed use.

upMyCounter := aMagmaSession serverRequest counterIncrementAt: #myKey.

.... persist in the db ....
upMyCounter := <.... obtain from the db ...>.
.... then to use.

10 timesRepeat: [ upMyCounter submit ].

3) 
Multiple requests

(aMagmaSession serverRequest counterIncrementAt: #myKey; counterIncrementAt: #myKey) submit

4)
Sequential Requests (needs MagmaServerBackdoorResponder)

This 'smalltalk code' is not compiled at all instead it is collected
as a series of message sends, posted to the server and played there.

((aMagmaSession current serverRequest login: 'admin' password: 'magma rocks') repositoryController isReadOnly) submit.
 
To add your own responders see example MagmaCounterResponder, just subclass me. Instances of me are per repository.

Gjallar users can use: Q2Session current serverRequest, In this case it does not remember the actual magmaSession to respond to, but the Process Local Variable which can be used to obtain the Q2Session current. This allows you to asseble a request persist it sendit around do what ever you wish with it, then when you want to send it it will automatically use the current session.!

!MagmaServerRequest class methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 02:01'!
submitTo: aMagmaSessionOrSimilar

	^  self basicNew setSubmitReceiver: aMagmaSessionOrSimilar ; initialize ! !

!MagmaServerRequest methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 01:30'!
addRequest: aMessage

	requests add: aMessage
	! !

!MagmaServerRequest methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 01:30'!
doesNotUnderstand: aMessage

	self addRequest: aMessage
	! !

!MagmaServerRequest methodsFor: 'as yet unclassified' stamp: 'kph 6/15/2007 02:29'!
initialize

  	requests := OrderedCollection new! !

!MagmaServerRequest methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 02:25'!
process

	 ^ MagmaRequestResponder responderFor: repositoryController playRequests: requests! !

!MagmaServerRequest methodsFor: 'as yet unclassified' stamp: 'kph 6/14/2007 23:08'!
requiresConnection

	^ true! !

!MagmaServerRequest methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 02:00'!
setSubmitReceiver: respondsToSubmit

	"the magmaSessionProvider is
		a) either a MagmaSession
		b) an objectwhich responds to the protocol we need i.e. #submit: "
 
	submitReceiver := respondsToSubmit
	! !

!MagmaServerRequest methodsFor: 'as yet unclassified' stamp: 'kph 6/16/2007 02:02'!
submit

	^ submitReceiver submit: self! !
MagmaRequestResponder initialize!
